@@ -155,7 +155,11 @@ def _get_statement():
 
     This is not intended for end-users but is useful in development.
     """
-    return session.friendly.tb_data.statement
+    _ = current_lang.translate
+    if not session.saved_info:
+        print(_("Nothing to show: no exception recorded."))
+        return
+    return session.friendly[-1].tb_data.statement
 
 
 def www(search=None, python=False):
@@ -163,11 +167,11 @@ def www(search=None, python=False):
     default browser, linking to a specific url.
 
     * If no arguments are provided, and no exception has been raised, this opens
-      Friendly-traceback documentation.
+      Friendly's documentation.
 
     * If no arguments are provided, and an exception has been raised, in most
       cases, an internet search will be done using the exception message as the
-      search string. In a few cases, Friendly-traceback will link to a
+      search string. In a few cases, Friendly will link to a
       specific site based on its determination of the cause of the exception.
 
     * If no search argument is provided, but ``python`` is specified to be ``True``,
@@ -177,7 +181,7 @@ def www(search=None, python=False):
 
         * If an exception has been raised, in most cases a search of the Python
           documentation will be done using the exception name as the search term.
-          In a few cases, Friendly-traceback will link to a
+          In a few cases, Friendly will link to a
           specific site based on its determination of the cause of the exception.
 
     * If a search (string) argument is provided, we have three possibilities:
@@ -185,7 +189,7 @@ def www(search=None, python=False):
       * If the ``python`` argument is ``True``, a search will be done in the
         Python documentation site; otherwise
       * if the search term contains the string "friendly" (independent of case),
-        Friendly-traceback documentation will open
+        Friendly documentation will open
       * otherwise, an internet search will be done using the search string provided.
     """
     import urllib
@@ -209,14 +213,10 @@ def www(search=None, python=False):
     quote = urllib.parse.quote  # noqa
 
     ddg_url = "https://duckduckgo.com?q="
-    friendly_url = "https://aroberge.github.io/friendly-docs/docs/html/"
+    friendly_url = "https://aroberge.github.io/friendly-traceback-docs/docs/html/"
     python_docs_url = _("https://docs.python.org/3/")
     python_search_url = _("https://docs.python.org/3/search.html?q=")
-    if session.saved_info:
-        info = session.saved_info[-1]
-    else:
-        info = None
-
+    info = session.saved_info[-1] if session.saved_info else None
     if search is None and not python:  # default search
         if info is None:
             url = friendly_url
@@ -243,7 +243,7 @@ def www(search=None, python=False):
 
     try:
         webbrowser.open_new_tab(url)
-    except Exception:
+    except Exception:  # noqa
         sys.stderr.write(_("The default web browser cannot be used for searching."))
 
 
