@@ -2,7 +2,7 @@
 
 The purpose of this file is to help during development.
 
-The idea is to silence internal exceptions raised by Friendly-traceback
+The idea is to silence internal exceptions raised by Friendly
 itself for most users by redirecting them here, and have them
 printed only when debugging mode is activated.
 """
@@ -11,7 +11,7 @@ import sys
 # DEBUG is set to True for me. It can also be set to True from __main__ or when
 # using the debug() command in the console.
 
-DEBUG = r"users\andre\github\friendly-traceback" in __file__.lower()
+DEBUG = r"users\andre\github\friendly" in __file__.lower()
 EXIT = False
 
 
