@@ -10,6 +10,7 @@ from ..my_gettext import current_lang, no_information, internal_error
 from .. import info_variables
 from .. import debug_helper
 from .. import token_utils
+from .. import utils
 
 convert_type = info_variables.convert_type
 MESSAGES_PARSERS = []
@@ -45,46 +46,6 @@ def _get_cause(value, frame, tb_data):
     return {"cause": no_information()}
 
 
-@add_message_parser
-def parse_can_only_concatenate(message, *_args):
-    _ = current_lang.translate
-    # example: can only concatenate str (not "int") to str
-    pattern = re.compile(
-        r"can only concatenate (\w+) \(not [\'\"](\w+)[\'\"]\) to (\w+)"
-    )
-    match = re.search(pattern, message)
-    if match is None:
-        return {}
-
-    # TODO: add hint if one of them could be converted to make this possible
-
-    cause = _(
-        "You tried to concatenate (add) two different types of objects:\n"
-        "{first} and {second}.\n"
-    ).format(first=convert_type(match.group(1)), second=convert_type(match.group(2)))
-    return {"cause": cause}
-
-
-@add_message_parser
-def parse_must_be_str(message, *_args):
-    _ = current_lang.translate
-    # python 3.6 version: must be str, not int
-    # example: can only concatenate str (not "int") to str
-
-    pattern = re.compile(r"must be str, not (\w+)")
-    match = re.search(pattern, message)
-    if match is None:
-        return {}
-
-    # TODO: add hint if one of them could be converted to make this possible
-    # also see if can be generalized
-    cause = _(
-        "You tried to concatenate (add) two different types of objects:\n"
-        "{first} and {second}.\n"
-    ).format(first=convert_type("str"), second=convert_type(match.group(1)))
-    return {"cause": cause}
-
-
 def _convert_str_to_number(obj_type1, obj_type2, frame, tb_data):
     """Determines if a suggestion should be made to convert a string to a
     number type; potentially useful for beginners that write programs
@@ -105,12 +66,11 @@ def _convert_str_to_number(obj_type1, obj_type2, frame, tb_data):
         return None, None
 
     all_objects = info_variables.get_all_objects(tb_data.bad_line, frame)["name, obj"]
-    # TODO: review this logic
     for name, obj in all_objects:
         if isinstance(obj, str):
             try:
                 convert(obj)
-            except Exception:
+            except Exception:  # noqa
                 continue
             break
     else:
@@ -127,6 +87,57 @@ def _convert_str_to_number(obj_type1, obj_type2, frame, tb_data):
     return cause, hint
 
 
+@add_message_parser
+def parse_can_only_concatenate(message, frame, tb_data):
+    _ = current_lang.translate
+    # example: can only concatenate str (not "int") to str
+    pattern = re.compile(
+        r"can only concatenate (\w+) \(not [\'\"](\w+)[\'\"]\) to (\w+)"
+    )
+    match = re.search(pattern, message)
+    if match is None:
+        return {}
+
+    obj_type1 = match.group(1)
+    obj_type2 = match.group(2)
+
+    cause = _(
+        "You tried to concatenate (add) two different types of objects:\n"
+        "{first} and {second}.\n"
+    ).format(first=convert_type(obj_type1), second=convert_type(obj_type2))
+    if obj_type1 == "str":
+        more_cause, possible_hint = _convert_str_to_number(
+            obj_type1, obj_type2, frame, tb_data
+        )
+        if more_cause is not None:
+            return {"cause": cause + more_cause, "suggest": possible_hint}
+    return {"cause": cause}
+
+
+@add_message_parser
+def parse_must_be_str(message, frame, tb_data):
+    _ = current_lang.translate
+    # python 3.6 version: must be str, not int
+    # example: can only concatenate str (not "int") to str
+
+    pattern = re.compile(r"must be str, not (\w+)")
+    match = re.search(pattern, message)
+    if match is None:
+        return {}
+
+    cause = _(
+        "You tried to concatenate (add) two different types of objects:\n"
+        "{first} and {second}.\n"
+    ).format(first=convert_type("str"), second=convert_type(match.group(1)))
+    if match.group(1) in ["int", "float", "complex"]:
+        more_cause, possible_hint = _convert_str_to_number(
+            "str", match.group(1), frame, tb_data
+        )
+        if more_cause is not None:
+            return {"cause": cause + more_cause, "suggest": possible_hint}
+    return {"cause": cause}
+
+
 @add_message_parser
 def parse_unsupported_operand_type(message, frame, tb_data):
     _ = current_lang.translate
@@ -139,9 +150,6 @@ def parse_unsupported_operand_type(message, frame, tb_data):
     if match is None:
         return {}
 
-    # TODO: look if can be done by converting from str to other type.
-    # think of adding hint if that is the case.
-
     all_objects = info_variables.get_all_objects(tb_data.bad_line, frame)["name, obj"]
     operator = match.group(1)
     obj_type1 = match.group(2)
@@ -190,7 +198,7 @@ def parse_unsupported_operand_type(message, frame, tb_data):
         )
         if "^" in operator:
             can_exponentiate = True
-            for name, obj in all_objects:
+            for _name, obj in all_objects:
                 if not hasattr(obj, "__pow__"):
                     can_exponentiate = False
                     break
@@ -212,7 +220,7 @@ def parse_unsupported_operand_type(message, frame, tb_data):
             first=convert_type(obj_type1),
             second=convert_type(obj_type2),
         )
-    elif operator == "** or pow()" or operator == "**=":
+    elif operator in ("** or pow()", "**="):
         cause = _(
             "You tried to exponentiate (raise to a power)\n"
             "using two incompatible types of objects:\n"
@@ -246,17 +254,24 @@ def parse_unsupported_operand_type(message, frame, tb_data):
 
 
 @add_message_parser
-def parse_order_comparison(message, *_args):
+def parse_order_comparison(message, frame, tb_data):
     _ = current_lang.translate
     # example: '<' not supported between instances of 'int' and 'str'
     pattern = re.compile(
         r"[\'\"](.+)[\'\"] not supported between instances of [\'\"](\w+)[\'\"] and [\'\"](\w+)[\'\"]"  # noqa
     )
-    # TODO: check if one is string that could be converted to number
     match = re.search(pattern, message)
     if match is None:
         return {}
 
+    if match.group(2) == match.group(3) == "complex":
+        hint = _("Complex numbers cannot be ordered.\n")
+        cause = _(
+            "You tried to do an order comparison ({operator})\n"
+            "between two complex numbers.\n"
+        ).format(operator=match.group(1))
+        return {"cause": cause, "suggest": hint}
+
     cause = _(
         "You tried to do an order comparison ({operator})\n"
         "between two incompatible types of objects:\n"
@@ -266,6 +281,20 @@ def parse_order_comparison(message, *_args):
         first=convert_type(match.group(2)),
         second=convert_type(match.group(3)),
     )
+
+    number = None
+    if match.group(2) in ["int", "float"] and match.group(3) == "str":
+        number = match.group(2)
+    elif match.group(3) in ["int", "float"] and match.group(2) == "str":
+        number = match.group(3)
+
+    if number is not None:
+        more_cause, possible_hint = _convert_str_to_number(
+            "str", number, frame, tb_data
+        )
+        if more_cause is not None:
+            return {"cause": cause + more_cause, "suggest": possible_hint}
+
     return {"cause": cause}
 
 
@@ -326,7 +355,7 @@ def does_not_support_item_assignment(message, *_args):
         "You tried change part of such an immutable object: {obj},\n"
         "most likely by using an indexing operation.\n"
     ).format(obj=convert_type(name))
-    if name == "tuple" or name == "set":
+    if name in ("tuple", "set"):
         hint = _("Did you mean to use a list?\n")
         cause += _("Perhaps you meant to use a list instead.\n")
     cause = {"cause": cause}
@@ -437,7 +466,7 @@ def x_is_not_callable(message, frame, tb_data):
                 fn_call = tb_data.bad_line.replace(obj_name, "", 1).strip()
                 if fn_call.startswith("(") and fn_call.endswith(")"):
                     break
-        except Exception:
+        except Exception:  # noqa
             continue
     else:
         return {"cause": cause}
@@ -458,10 +487,8 @@ def x_is_not_callable(message, frame, tb_data):
     ).format(fn_call=fn_call, obj_name=obj_name, obj_type=obj_type)
 
     try:
-        # As a TypeError exception has been raised, Python has already evaluated
-        # all the relevant code parts. Thus, using eval() should be completely safe..
-        can_eval = eval(fn_call, frame.f_globals, frame.f_locals)
-    except Exception:
+        can_eval = utils.eval_expr(fn_call, frame)
+    except Exception:  # noqa
         return {"cause": cause}
 
     if isinstance(can_eval, tuple):
@@ -477,7 +504,7 @@ def x_is_not_callable(message, frame, tb_data):
         ).format(fn_call=fn_call, obj_name=obj_name)
         return {"cause": cause, "suggest": hint}
 
-    elif hasattr(obj, "__getitem__") and isinstance(can_eval, int):
+    if hasattr(obj, "__getitem__") and isinstance(can_eval, int):
         cause = cause + _(
             "However, `{obj_name}` is a sequence.\n"
             "Perhaps you meant to use `[]` instead of `()` and write\n"
@@ -488,7 +515,7 @@ def x_is_not_callable(message, frame, tb_data):
         )
         return {"cause": cause, "suggest": hint}
 
-    elif (  # Many objects can be multiplied, but only numbers should have __abs__
+    if (  # Many objects can be multiplied, but only numbers should have __abs__
         hasattr(obj, "__abs__")  # Should identify numbers: int, float, ...
         and hasattr(can_eval, "__abs__")  # complex, Fractions, Decimals, ...
         and hasattr(obj, "__mul__")  # Confirming that they can be multiplied
@@ -535,15 +562,16 @@ def cannot_multiply_by_str(message, frame, tb_data):
         tokens = token_utils.get_significant_tokens(tb_data.bad_line)
         int_vars = []
         for prev_token, token in zip(tokens, tokens[1:]):
-            if prev_token.is_in(["*", "*="]) and token.is_in(names):
+            if prev_token.string in ("*", "*=") and token.string in names:
                 int_vars.append(token.string)
-            elif prev_token.is_in(names) and token == "*":
+            elif prev_token.string in names and token == "*":
                 int_vars.append(prev_token.string)
             else:
                 continue
         if not int_vars:  # should not happen, but better be safe
             return {"cause": cause}
-        elif len(int_vars) == 1:
+
+        if len(int_vars) == 1:
             more_cause, hint = forgot_to_convert_name_to_int(int_vars[0])
             cause += more_cause
         else:
@@ -566,7 +594,7 @@ def find_possible_integers(object_of_type, frame, line):
             try:
                 int(obj)
                 names.append(name)
-            except Exception:
+            except Exception:  # noqa
                 pass
 
     return names
@@ -648,10 +676,8 @@ def indices_must_be_integers_or_slices(message, frame, tb_data):
     #    do we get a valid expression.
     # 2. if ... is something of another type that can be converted into an integer
     try:
-        # By this stage, Python should already have evaluated container_type.
-        # Thus, using eval() should be completely safe.
-        container_type = eval(container_type, frame.f_globals, frame.f_locals)
-    except Exception:
+        container_type = utils.eval_expr(container_type, frame)
+    except Exception:  # noqa
         if additional_cause:
             return {"cause": cause + additional_cause, "suggest": hint}
         return {"cause": cause}
@@ -689,11 +715,9 @@ def indices_must_be_integers_or_slices(message, frame, tb_data):
     index = index[1:-1]
 
     try:
-        # As a TypeError exception has been raised, Python has already evaluated
-        # all the relevant code parts. Thus, using eval() should be completely safe.
-        index = eval(index, frame.f_globals, frame.f_locals)
-        index_type = eval(index_type)
-    except Exception:
+        index = utils.eval_expr(index, frame)
+        index_type = utils.eval_expr(index_type, frame)
+    except Exception:  # noqa
         if additional_cause:
             return {"cause": cause + additional_cause, "suggest": hint}
         return {"cause": cause}
@@ -711,10 +735,8 @@ def indices_must_be_integers_or_slices(message, frame, tb_data):
             .replace(" ", "")
         )
         try:
-            # As a TypeError exception has been raised, Python has already evaluated
-            # all the relevant code parts. Thus, using eval() should be completely safe.
-            result = [] == eval(newline, frame.f_globals, frame.f_locals)
-        except Exception:
+            result = [] == utils.eval_expr(newline, frame)
+        except Exception:  # noqa
             result = False
 
         if not result:
@@ -729,14 +751,14 @@ def indices_must_be_integers_or_slices(message, frame, tb_data):
             line=container + newline.replace("[]", "", 1)
         )
         return {"cause": cause + "\n" + additional_cause, "suggest": hint}
-    else:
-        names = find_possible_integers(index_type, frame, tb_data.bad_line)
-        if len(names) == 1:  # This should usually be the case
-            more_cause, hint = forgot_to_convert_name_to_int(names[0])
-            cause += "\n" + more_cause
-            if hint is not None:
-                return {"cause": cause, "suggest": hint}
-            return {"cause": cause}
+
+    names = find_possible_integers(index_type, frame, tb_data.bad_line)
+    if len(names) == 1:  # This should usually be the case
+        more_cause, hint = forgot_to_convert_name_to_int(names[0])
+        cause += "\n" + more_cause
+        if hint is not None:
+            return {"cause": cause, "suggest": hint}
+        return {"cause": cause}
 
     if additional_cause:
         return {"cause": cause + additional_cause, "suggest": hint}
